@@ -17,20 +17,23 @@
 # @author: Stéphane Albert
 #
 import decimal
+import random
+import uuid
 
 import eventlet
-from oslo.config import cfg
 from oslo_concurrency import lockutils
+from oslo_config import cfg
 from oslo_log import log as logging
 import oslo_messaging as messaging
-import six
 from stevedore import driver
-from stevedore import extension
+from tooz import coordination
 
 from cloudkitty import collector
 from cloudkitty.common import rpc
 from cloudkitty import config  # noqa
 from cloudkitty import extension_manager
+from cloudkitty import storage
+from cloudkitty import transformer
 from cloudkitty import utils as ck_utils
 
 eventlet.monkey_patch()
@@ -38,21 +41,26 @@ eventlet.monkey_patch()
 LOG = logging.getLogger(__name__)
 
 CONF = cfg.CONF
-CONF.import_opt('backend', 'cloudkitty.storage', 'storage')
 CONF.import_opt('backend', 'cloudkitty.tenant_fetcher', 'tenant_fetcher')
 
-COLLECTORS_NAMESPACE = 'cloudkitty.collector.backends'
+orchestrator_opts = [
+    cfg.StrOpt('coordination_url',
+               secret=True,
+               help='Coordination driver URL',
+               default='file:///var/lib/cloudkitty/locks'),
+]
+CONF.register_opts(orchestrator_opts, group='orchestrator')
+
 FETCHERS_NAMESPACE = 'cloudkitty.tenant.fetchers'
-TRANSFORMERS_NAMESPACE = 'cloudkitty.transformers'
 PROCESSORS_NAMESPACE = 'cloudkitty.rating.processors'
-STORAGES_NAMESPACE = 'cloudkitty.storage.backends'
 
 
 class RatingEndpoint(object):
     target = messaging.Target(namespace='rating',
-                              version='1.0')
+                              version='1.1')
 
     def __init__(self, orchestrator):
+        self._global_reload = False
         self._pending_reload = []
         self._module_state = {}
         self._orchestrator = orchestrator
@@ -76,21 +84,27 @@ class RatingEndpoint(object):
         worker = APIWorker()
         return str(worker.quote(res_data))
 
+    def reload_modules(self, ctxt):
+        LOG.info('Received reload modules command.')
+        lock = lockutils.lock('module-reload')
+        with lock:
+            self._global_reload = True
+
     def reload_module(self, ctxt, name):
-        LOG.info('Received reload command for module {}.'.format(name))
+        LOG.info('Received reload command for module %s.', name)
         lock = lockutils.lock('module-reload')
         with lock:
             if name not in self._pending_reload:
                 self._pending_reload.append(name)
 
     def enable_module(self, ctxt, name):
-        LOG.info('Received enable command for module {}.'.format(name))
+        LOG.info('Received enable command for module %s.', name)
         lock = lockutils.lock('module-state')
         with lock:
             self._module_state[name] = True
 
     def disable_module(self, ctxt, name):
-        LOG.info('Received disable command for module {}.'.format(name))
+        LOG.info('Received disable command for module %s.', name)
         lock = lockutils.lock('module-state')
         with lock:
             self._module_state[name] = False
@@ -110,8 +124,7 @@ class BaseWorker(object):
         self._processors = []
         processors = extension_manager.EnabledExtensionManager(
             PROCESSORS_NAMESPACE,
-            invoke_kwds={'tenant_id': self._tenant_id}
-        )
+            invoke_kwds={'tenant_id': self._tenant_id})
 
         for processor in processors:
             self._processors.append(processor)
@@ -130,7 +143,8 @@ class APIWorker(BaseWorker):
         for res in res_data:
             for res_usage in res['usage'].values():
                 for data in res_usage:
-                    price += data.get('rating', {}).get('price', 0.0)
+                    price += data.get('rating', {}).get('price',
+                                                        decimal.Decimal(0))
         return price
 
 
@@ -150,11 +164,10 @@ class Worker(BaseWorker):
                                             start_timestamp,
                                             next_timestamp,
                                             self._tenant_id)
-
-        timed_data = [{'period': {'begin': start_timestamp,
-                                  'end': next_timestamp},
-                      'usage': raw_data}]
-        return timed_data
+        if raw_data:
+            return [{'period': {'begin': start_timestamp,
+                                'end': next_timestamp},
+                     'usage': raw_data}]
 
     def check_state(self):
         timestamp = self._storage.get_state(self._tenant_id)
@@ -181,9 +194,9 @@ class Worker(BaseWorker):
                     except collector.NoDataCollected:
                         raise
                     except Exception as e:
-                        LOG.warn('Error while collecting service {service}:'
-                                 ' {error}'.format(service=service,
-                                                   error=six.text_type(e)))
+                        LOG.warning(
+                            'Error while collecting service %(service)s: '
+                            '%(error)s', {'service': service, 'error': e})
                         raise collector.NoDataCollected('', service)
                 except collector.NoDataCollected:
                     begin = timestamp
@@ -210,32 +223,28 @@ class Orchestrator(object):
             CONF.tenant_fetcher.backend,
             invoke_on_load=True).driver
 
-        # Transformers
-        self.transformers = {}
-        self._load_transformers()
-
-        collector_args = {'transformers': self.transformers,
-                          'period': CONF.collect.period}
-        self.collector = driver.DriverManager(
-            COLLECTORS_NAMESPACE,
-            CONF.collect.collector,
-            invoke_on_load=True,
-            invoke_kwds=collector_args).driver
-
-        storage_args = {'period': CONF.collect.period}
-        self.storage = driver.DriverManager(
-            STORAGES_NAMESPACE,
-            CONF.storage.backend,
-            invoke_on_load=True,
-            invoke_kwds=storage_args).driver
+        self.transformers = transformer.get_transformers()
+        self.collector = collector.get_collector(self.transformers)
+        self.storage = storage.get_storage(self.collector)
 
         # RPC
         self.server = None
         self._rating_endpoint = RatingEndpoint(self)
         self._init_messaging()
 
+        # DLM
+        self.coord = coordination.get_coordinator(
+            CONF.orchestrator.coordination_url,
+            str(uuid.uuid4()).encode('ascii'))
+        self.coord.start()
+
+    def _lock(self, tenant_id):
+        lock_name = b"cloudkitty-" + str(tenant_id).encode('ascii')
+        return self.coord.get_lock(lock_name)
+
     def _load_tenant_list(self):
         self._tenants = self.fetcher.get_tenants()
+        random.shuffle(self._tenants)
 
     def _init_messaging(self):
         target = messaging.Target(topic='cloudkitty',
@@ -260,28 +269,6 @@ class Orchestrator(object):
             return next_timestamp
         return 0
 
-    def _collect(self, service, start_timestamp):
-        next_timestamp = start_timestamp + CONF.collect.period
-        raw_data = self.collector.retrieve(service,
-                                           start_timestamp,
-                                           next_timestamp)
-
-        timed_data = [{'period': {'begin': start_timestamp,
-                                  'end': next_timestamp},
-                      'usage': raw_data}]
-        return timed_data
-
-    def _load_transformers(self):
-        self.transformers = {}
-        transformers = extension.ExtensionManager(
-            TRANSFORMERS_NAMESPACE,
-            invoke_on_load=True)
-
-        for transformer in transformers:
-            t_name = transformer.name
-            t_obj = transformer.obj
-            self.transformers[t_name] = t_obj
-
     def process_messages(self):
         # TODO(sheeprine): Code kept to handle threading and asynchronous
         # reloading
@@ -294,16 +281,23 @@ class Orchestrator(object):
             self.process_messages()
             self._load_tenant_list()
             while len(self._tenants):
-                for tenant in self._tenants:
-                    if not self._check_state(tenant):
-                        self._tenants.remove(tenant)
-                    else:
-                        worker = Worker(self.collector,
-                                        self.storage,
-                                        tenant)
-                        worker.run()
+                for tenant in self._tenants[:]:
+                    lock = self._lock(tenant)
+                    if lock.acquire(blocking=False):
+                        if not self._check_state(tenant):
+                            self._tenants.remove(tenant)
+                        else:
+                            worker = Worker(self.collector,
+                                            self.storage,
+                                            tenant)
+                            worker.run()
+                        lock.release()
+                    self.coord.heartbeat()
+                # NOTE(sheeprine): Slow down looping if all tenants are
+                # being processed
+                eventlet.sleep(1)
             # FIXME(sheeprine): We may cause a drift here
             eventlet.sleep(CONF.collect.period)
 
     def terminate(self):
-        pass
+        self.coord.stop()
