@@ -32,7 +32,15 @@ def get_instance():
     return IMPL
 
 
-class NoSuchService(Exception):
+class BaseHashMapError(Exception):
+    """Base class for HashMap errors."""
+
+
+class ClientHashMapError(BaseHashMapError):
+    """Base class for client side errors."""
+
+
+class NoSuchService(ClientHashMapError):
     """Raised when the service doesn't exist."""
 
     def __init__(self, name=None, uuid=None):
@@ -42,7 +50,7 @@ class NoSuchService(Exception):
         self.uuid = uuid
 
 
-class NoSuchField(Exception):
+class NoSuchField(ClientHashMapError):
     """Raised when the field doesn't exist for the service."""
 
     def __init__(self, uuid):
@@ -51,7 +59,7 @@ class NoSuchField(Exception):
         self.uuid = uuid
 
 
-class NoSuchGroup(Exception):
+class NoSuchGroup(ClientHashMapError):
     """Raised when the group doesn't exist."""
 
     def __init__(self, name=None, uuid=None):
@@ -61,7 +69,7 @@ class NoSuchGroup(Exception):
         self.uuid = uuid
 
 
-class NoSuchMapping(Exception):
+class NoSuchMapping(ClientHashMapError):
     """Raised when the mapping doesn't exist."""
 
     def __init__(self, uuid):
@@ -70,7 +78,7 @@ class NoSuchMapping(Exception):
         self.uuid = uuid
 
 
-class NoSuchThreshold(Exception):
+class NoSuchThreshold(ClientHashMapError):
     """Raised when the threshold doesn't exist."""
 
     def __init__(self, uuid):
@@ -79,7 +87,7 @@ class NoSuchThreshold(Exception):
         self.uuid = uuid
 
 
-class NoSuchType(Exception):
+class NoSuchType(ClientHashMapError):
     """Raised when a mapping type is not handled."""
 
     def __init__(self, map_type):
@@ -89,7 +97,7 @@ class NoSuchType(Exception):
         self.map_type = map_type
 
 
-class ServiceAlreadyExists(Exception):
+class ServiceAlreadyExists(ClientHashMapError):
     """Raised when the service already exists."""
 
     def __init__(self, name, uuid):
@@ -99,7 +107,7 @@ class ServiceAlreadyExists(Exception):
         self.uuid = uuid
 
 
-class FieldAlreadyExists(Exception):
+class FieldAlreadyExists(ClientHashMapError):
     """Raised when the field already exists."""
 
     def __init__(self, field, uuid):
@@ -109,7 +117,7 @@ class FieldAlreadyExists(Exception):
         self.uuid = uuid
 
 
-class GroupAlreadyExists(Exception):
+class GroupAlreadyExists(ClientHashMapError):
     """Raised when the group already exists."""
 
     def __init__(self, name, uuid):
@@ -119,27 +127,38 @@ class GroupAlreadyExists(Exception):
         self.uuid = uuid
 
 
-class MappingAlreadyExists(Exception):
+class MappingAlreadyExists(ClientHashMapError):
     """Raised when the mapping already exists."""
 
-    def __init__(self, mapping, uuid):
+    def __init__(self, mapping, parent_id=None, parent_type=None, uuid=None):
+        # TODO(sheeprine): UUID is deprecated
+        parent_id = parent_id if parent_id else uuid
         super(MappingAlreadyExists, self).__init__(
-            "Mapping %s already exists (UUID: %s)" % (mapping, uuid))
+            "Mapping '%s' already exists for %s '%s'" % (mapping,
+                                                         parent_type,
+                                                         parent_id))
         self.mapping = mapping
-        self.uuid = uuid
+        self.uuid = parent_id
+        self.parent_id = parent_id
+        self.parent_type = parent_type
 
 
-class ThresholdAlreadyExists(Exception):
+class ThresholdAlreadyExists(ClientHashMapError):
     """Raised when the threshold already exists."""
 
-    def __init__(self, threshold, uuid):
+    def __init__(self, threshold, parent_id=None, parent_type=None, uuid=None):
+        # TODO(sheeprine): UUID is deprecated
+        parent_id = parent_id if parent_id else uuid
         super(ThresholdAlreadyExists, self).__init__(
-            "Threshold %s already exists (UUID: %s)" % (threshold, uuid))
+            "Threshold '%s' already exists for %s '%s'"
+            % (threshold, parent_type, parent_id))
         self.threshold = threshold
-        self.uuid = uuid
+        self.uuid = parent_id
+        self.parent_id = parent_id
+        self.parent_type = parent_type
 
 
-class MappingHasNoGroup(Exception):
+class MappingHasNoGroup(ClientHashMapError):
     """Raised when the mapping is not attached to a group."""
 
     def __init__(self, uuid):
@@ -148,7 +167,7 @@ class MappingHasNoGroup(Exception):
         self.uuid = uuid
 
 
-class ThresholdHasNoGroup(Exception):
+class ThresholdHasNoGroup(ClientHashMapError):
     """Raised when the threshold is not attached to a group."""
 
     def __init__(self, uuid):
@@ -185,10 +204,11 @@ class HashMap(object):
         """
 
     @abc.abstractmethod
-    def get_group(self, uuid):
+    def get_group(self, uuid=None, name=None):
         """Return a group object.
 
         :param uuid: UUID of the group to get.
+        :param name: Name of the group to get.
         """
 
     @abc.abstractmethod
