@@ -18,7 +18,7 @@
 #
 import abc
 
-from oslo.config import cfg
+from oslo_config import cfg
 import six
 
 fetchers_opts = [
